﻿Imports System.Text.RegularExpressions


''' <summary>
''' PhysicalUnit is a helper class that allows you to convert the inputed strings to 
''' a Voltage or Current (V or A).
''' </summary>
''' <remarks></remarks>
Public Class PhysicalUnit

    Private _text As String = ""


    ' property to set/retrieve the string 
    Public Property text As String
        Get
            Return _text
        End Get
        Set(value As String)
            _text = value.ToUpper().Trim()
        End Set
    End Property


    ''' <summary>
    ''' Constructor of the class. You need to pass the complete string to convert
    ''' </summary>
    ''' <param name="text"></param>
    ''' <remarks></remarks>
    Public Sub New(ByVal text As String)
        Me.text = text
    End Sub


    ''' <summary>
    ''' extractValue() extracts the value from the string. The result is stored in the
    ''' result variable (passed as byRef).
    ''' </summary>
    ''' <param name="result">The result of the conversion</param>
    ''' <param name="unit">The unit of the conversion</param>
    ''' <returns>A boolean to indicate if the function succeeded</returns>
    ''' <remarks></remarks>
    Public Function extractValue(ByRef result As Double, ByVal unit As Char) As Boolean

        Dim numberToConvert As String
        Dim multiplier As Integer

        If isNumberOfUnit(unit) Then
            If Right(text, 2) = "M" & unit Then
                numberToConvert = text.Replace("M" & unit, "")
                multiplier = 1
            Else
                numberToConvert = text.Replace(unit, "")
                multiplier = 1000
            End If

            If Double.TryParse(numberToConvert, result) Then
                result *= multiplier
                Return True
            Else
                Return False
            End If
        Else
            Return False
        End If


    End Function



    ''' <summary>
    ''' isNumberOfUnit() validates the number. It verifies if the string matches the
    ''' required pattern.
    ''' 
    ''' 10.2mV would return true (if the unit is set to V)
    ''' 
    ''' </summary>
    ''' <param name="unit">The unit: V or A</param>
    ''' <returns>A boolean to indicate if the string format is valid</returns>
    ''' <remarks></remarks>
    Public Function isNumberOfUnit(ByVal unit As Char)

        Dim decimalSeparator As String = Globalization.CultureInfo.CurrentCulture.NumberFormat.NumberDecimalSeparator

        Dim reg As String = "^[0-9]+(\{separator}[0-9]+)?[Mm]?%unit%$"
        reg = reg.Replace("{separator}", decimalSeparator)

        reg = reg.Replace("%unit%", unit)

        If Regex.IsMatch(_text, reg) Then
            Return True
        Else
            Return False
        End If

    End Function




End Class
